#include "ChaconController.h"

#define NUMBER_SEQUENCE_REPEATS 3

#define CODE_BIG_MARKER 1
#define CODE_SMALL_MARKER 2
#define CODE_ONE_MARKER 3
#define CODE_ZERO_MARKER 4

#define DURATION_BIG_MARKER_HIGH 350
#define DURATION_BIG_MARKER_LOW 10000

#define DURATION_SMALL_MARKER_HIGH 350
#define DURATION_SMALL_MARKER_LOW 2600

#define DURATION_ZERO_MARKER_HIGH 350
#define DURATION_ZERO_MARKER_LOW 190

#define DURATION_ONE_MARKER_HIGH 350
#define DURATION_ONE_MARKER_LOW 1190

// Initialize the controller with the address (3 bytes)
ChaconController::ChaconController(byte data_pin, byte a2, byte a1, byte a0)
{
	this->data_pin = data_pin;
	this->a2 = a2;
	this->a1 = a1;
	this->a0 = a0;
	this->end_last_code = 0;
}

void ChaconController::sendCommand(byte device, byte state)
{
    #if CHACON_DEBUG_MODE>2
    Serial.print("Sending: device:");
    Serial.print(device, DEC);
    Serial.print("  state:");
    Serial.println(state, DEC);
    #endif
    end_last_code = micros();
    digitalWrite(data_pin, HIGH);
    // Sends the signal NUMBER_SEQUENCE_REPEATS times
    for (int i=0; i<NUMBER_SEQUENCE_REPEATS; i++)
    {
		// Sends the marks that indicate the beginning of a message
        sendMark(CODE_BIG_MARKER);
        sendMark(CODE_SMALL_MARKER);
		// Sends the address of the emitter
        sendByte(a2);
        sendByte(a1);
        sendByte(a0);
        byte databyte;
		// 144 and 128 are the byte codes to turn ON and OFF
        if (state) databyte = 144;
        else databyte = 128;
		// databyte is merged with the device id so it activates the right device
        sendByte(databyte | device);
    }
    digitalWrite(data_pin, LOW);    
}

// Functions to send data
void ChaconController::sendMark(byte code)
{
    unsigned long end_time_high;
    unsigned long end_time_low;
    switch(code)
    {
        case CODE_BIG_MARKER:   end_time_high = end_last_code + DURATION_BIG_MARKER_HIGH; 
                                end_time_low = end_time_high + DURATION_BIG_MARKER_LOW; break;
                                
        case CODE_SMALL_MARKER: end_time_high = end_last_code + DURATION_SMALL_MARKER_HIGH;
                                end_time_low = end_time_high + DURATION_SMALL_MARKER_LOW; break;
                                
        case CODE_ONE_MARKER:   end_time_high = end_last_code + DURATION_ONE_MARKER_HIGH;
                                end_time_low = end_time_high + DURATION_ONE_MARKER_LOW; break;
                                
        case CODE_ZERO_MARKER:  end_time_high = end_last_code + DURATION_ZERO_MARKER_HIGH;
                                end_time_low = end_time_high + DURATION_ZERO_MARKER_LOW; break;
    }
    while (micros()<end_time_high);
        digitalWrite(data_pin, LOW);    
    while (micros()<end_time_low);
        digitalWrite(data_pin, HIGH);    
    end_last_code = end_time_low;
}

void ChaconController::sendByte(byte next_byte)
{
    for (int i=0; i<8; i++)
    {
        if (next_byte & (0x80 >> i) ) 
        {
            sendMark(CODE_ONE_MARKER);
            sendMark(CODE_ZERO_MARKER);
        }
        else
        {
            sendMark(CODE_ZERO_MARKER);
            sendMark(CODE_ONE_MARKER);
        }
    }
}


