
//#include "WProgram.h"
#include "Clock.h"

#include <Wire.h>


void getTime(byte *hours, byte *minutes, byte *seconds)
{
	Wire.beginTransmission(DS1307_ADDRESS);
	Wire.send(0);
	Wire.endTransmission();

	Wire.requestFrom(DS1307_ADDRESS, 3);

	*seconds = bcdToDec(Wire.receive());
	*minutes = bcdToDec(Wire.receive());
	*hours = bcdToDec(Wire.receive() & 0b111111); //24 hour time
}

long getDaySeconds()
{
	int hours, minutes, seconds;
	Wire.beginTransmission(DS1307_ADDRESS);
	Wire.send(0);
	Wire.endTransmission();

	Wire.requestFrom(DS1307_ADDRESS, 3);

	seconds = bcdToDec(Wire.receive());
	minutes = bcdToDec(Wire.receive());
	hours = bcdToDec(Wire.receive() & 0b111111); //24 hour time
	return 3600* (long)hours + 60*minutes + seconds;
}

long timeToSeconds(byte *hours, byte *minutes, byte *seconds)
{
	return 3600*(long)(*hours)+60*(*minutes)+(*seconds);
}

void secondsToTime(long *long_seconds, byte *hours, byte *minutes, byte *seconds)
{
	*hours = (*long_seconds)/3600;
	int aux = (*long_seconds)%3600;
	*minutes = aux/60;
	*seconds = aux%60;
}
char* itoaFill(byte *value, char *buff)
{
	if ( *value<10)
	{
		buff[0] = '0';
		buff[1] = *value + '0';
	}
	else
		itoa(*value, buff, 10);
	return buff;
}
	
void timeString(char *buff, byte *hours, byte *minutes, byte *seconds)
{
	itoaFill(hours, buff);
	buff[2] = ':';
	itoaFill(minutes, &buff[3]);
	buff[5] = ':';
	itoaFill(seconds, &buff[6]);
	buff[8] = 0;
}

void timeString(char *buff, long *long_seconds)
{
	byte hours, minutes, seconds;
	secondsToTime(long_seconds, &hours, &minutes, &seconds);
	timeString(buff, &hours, &minutes, &seconds);
}

void timeString(char *buff)
{
	byte hours, minutes, seconds;
	getTime(&hours, &minutes, &seconds);
	timeString(buff, &hours, &minutes, &seconds);
}

void getTimeFromString(char *buff, byte *hours, byte *minutes, byte *seconds)
{
	char buf[3];
	buf[0] = buff[0];
	buf[1] = buff[1];
	buf[2] = 0;
	*hours = atoi(buf);
	*hours %= 24;
	buf[0] = buff[3];
	buf[1] = buff[4];
	*minutes = atoi(buf);
	*minutes %= 60;
	buf[0] = buff[6];
	buf[1] = buff[7];
	*seconds = atoi(buf);
	*seconds %= 60;
}

void getDate(byte *day, byte *month, byte *year)
{
	// Reset the register pointer
	Wire.beginTransmission(DS1307_ADDRESS);
	Wire.send(4);
	Wire.endTransmission();

	Wire.requestFrom(DS1307_ADDRESS, 3);

	*day = bcdToDec(Wire.receive());
	*month = bcdToDec(Wire.receive());
	*year = bcdToDec(Wire.receive());
}

void setClock(byte hours, byte minutes, byte seconds, byte day, byte month, byte year, byte week_day)
{
	Wire.beginTransmission(DS1307_ADDRESS);
	Wire.send(0); //stop Oscillator

	Wire.send(decToBcd(seconds));
	Wire.send(decToBcd(minutes));
	Wire.send(decToBcd(hours));
	Wire.send(decToBcd(week_day));
	Wire.send(decToBcd(day));
	Wire.send(decToBcd(month));
	Wire.send(decToBcd(year));

	Wire.send(0); //start 

	Wire.endTransmission();
}

byte bcdToDec(byte val)
{
	// Convert binary coded decimal to normal decimal numbers
	return ( (val/16*10) + (val%16) );
}

byte decToBcd(byte val)
{
// Convert normal decimal numbers to binary coded decimal
  return ( (val/10*16) + (val%10) );
}
